@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE PackageImports #-}
 -- |
 -- Module      : Crypto.Hash
 -- License     : BSD-style
@@ -33,27 +34,27 @@ module Crypto.Hash
     , hashUpdate
     , hashInitAlg
     -- * hash algorithms
-    , MD2(..)
-    , MD4(..)
-    , MD5(..)
-    , SHA1(..)
-    , SHA224(..)
-    , SHA256(..)
-    , SHA384(..)
-    , SHA512(..)
-    , RIPEMD160(..)
-    , Tiger(..)
-    , SHA3_224(..)
-    , SHA3_256(..)
-    , SHA3_384(..)
-    , SHA3_512(..)
-    , Skein256_224(..)
-    , Skein256_256(..)
-    , Skein512_224(..)
-    , Skein512_256(..)
-    , Skein512_384(..)
-    , Skein512_512(..)
-    , Whirlpool(..)
+    , H.MD2(..)
+    , H.MD4(..)
+    , H.MD5(..)
+    , H.SHA1(..)
+    , H.SHA224(..)
+    , H.SHA256(..)
+    , H.SHA384(..)
+    , H.SHA512(..)
+    , H.RIPEMD160(..)
+    , H.Tiger(..)
+    , H.SHA3_224(..)
+    , H.SHA3_256(..)
+    , H.SHA3_384(..)
+    , H.SHA3_512(..)
+    , H.Skein256_224(..)
+    , H.Skein256_256(..)
+    , H.Skein512_224(..)
+    , H.Skein512_256(..)
+    , H.Skein512_384(..)
+    , H.Skein512_512(..)
+    , H.Whirlpool(..)
     -- * MAC algorithms
     , HMAC(..)
     , hmac
@@ -61,27 +62,14 @@ module Crypto.Hash
     ) where
 
 import Crypto.Hash.Types
-import Crypto.Hash.Utils
 import Data.ByteString (ByteString)
 import Data.Byteable
 import Data.Bits (xor)
 import qualified Data.ByteString as B
+import qualified Data.ByteArray.Encoding as B
 import qualified Data.ByteString.Lazy as L
 
-import qualified Crypto.Hash.MD2 as MD2
-import qualified Crypto.Hash.MD4 as MD4
-import qualified Crypto.Hash.MD5 as MD5
-import qualified Crypto.Hash.SHA1 as SHA1
-import qualified Crypto.Hash.SHA224 as SHA224
-import qualified Crypto.Hash.SHA256 as SHA256
-import qualified Crypto.Hash.SHA384 as SHA384
-import qualified Crypto.Hash.SHA512 as SHA512
-import qualified Crypto.Hash.SHA3 as SHA3
-import qualified Crypto.Hash.RIPEMD160 as RIPEMD160
-import qualified Crypto.Hash.Tiger as Tiger
-import qualified Crypto.Hash.Skein256 as Skein256
-import qualified Crypto.Hash.Skein512 as Skein512
-import qualified Crypto.Hash.Whirlpool as Whirlpool
+import qualified "cryptonite" Crypto.Hash as H
 
 -- | Alias to a single pass hash function that operate on a strict bytestring
 type HashFunctionBS a = ByteString -> Digest a
@@ -103,26 +91,53 @@ hashlazy lbs = hashFinalize $ hashUpdates hashInit (L.toChunks lbs)
 
 -- | Return the hexadecimal (base16) bytestring of the digest
 digestToHexByteString :: Digest a -> ByteString
-digestToHexByteString = toHex . toBytes
+digestToHexByteString = B.convertToBase B.Base16 . toBytes
+
+-- | Class representing hashing algorithms.
+--
+-- The hash algorithm is built over 3 primitives:
+--
+-- * init     : create a new hashing context
+--
+-- * updates  : update the hashing context with some strict bytestrings
+--              and return the new context
+--
+-- * finalize : finalize the context into a digest
+--
+class HashAlgorithm a where
+    -- | Block size in bytes the hash algorithm operates on
+    hashBlockSize :: Context a -> Int
+
+    -- | Initialize a new context for this hash algorithm
+    hashInit     :: Context a
+
+    -- | Update the context with a list of strict bytestring,
+    -- and return a new context with the updates.
+    hashUpdates  :: Context a -> [ByteString] -> Context a
+
+    -- | Finalize a context and return a digest.
+    hashFinalize :: Context a -> Digest a
+
+    -- | Try to convert a binary digest bytestring to a digest.
+    digestFromByteString :: ByteString -> Maybe (Digest a)
+
 
 #define DEFINE_INSTANCE(NAME, MODULENAME, BLOCKSIZE) \
-data NAME = NAME deriving Show; \
-instance HashAlgorithm NAME where \
-    { hashInit = Context c where { (MODULENAME.Ctx c) = MODULENAME.init } \
+instance HashAlgorithm H.NAME where \
+    { hashInit = Context $ H.hashInit \
     ; hashBlockSize ~(Context _) = BLOCKSIZE \
-    ; hashUpdates (Context c) bs = Context nc where { (MODULENAME.Ctx nc) = MODULENAME.updates (MODULENAME.Ctx c) bs } \
-    ; hashFinalize (Context c) = Digest $ MODULENAME.finalize (MODULENAME.Ctx c) \
-    ; digestFromByteString bs = if B.length bs == len then (Just $ Digest bs) else Nothing where { len = B.length (MODULENAME.finalize MODULENAME.init) } \
+    ; hashUpdates (Context c) bs = Context $ H.hashUpdates c bs \
+    ; hashFinalize (Context c) = Digest $ H.hashFinalize c \
+    ; digestFromByteString bs = Digest `fmap` H.digestFromByteString bs \
     };
 
 #define DEFINE_INSTANCE_LEN(NAME, MODULENAME, LEN, BLOCKSIZE) \
-data NAME = NAME deriving Show; \
-instance HashAlgorithm NAME where \
-    { hashInit = Context c where { (MODULENAME.Ctx c) = MODULENAME.init LEN } \
+instance HashAlgorithm H.NAME where \
+    { hashInit = Context $ H.hashInit \
     ; hashBlockSize ~(Context _) = BLOCKSIZE \
-    ; hashUpdates (Context c) bs = Context nc where { (MODULENAME.Ctx nc) = MODULENAME.updates (MODULENAME.Ctx c) bs } \
-    ; hashFinalize (Context c) = Digest $ MODULENAME.finalize (MODULENAME.Ctx c) \
-    ; digestFromByteString bs = if B.length bs == len then (Just $ Digest bs) else Nothing where { len = B.length (MODULENAME.finalize (MODULENAME.init LEN)) } \
+    ; hashUpdates (Context c) bs = Context $ H.hashUpdates c bs \
+    ; hashFinalize (Context c) = Digest $ H.hashFinalize c \
+    ; digestFromByteString bs = Digest `fmap` H.digestFromByteString bs \
     };
 
 -- | MD2 cryptographic hash
