@@ -97,14 +97,14 @@ static inline void sha3_do_chunk(uint64_t state[25], uint64_t buf[], int bufsz)
 	}
 }
 
-void sha3_init(struct sha3_ctx *ctx, uint32_t hashlen)
+void cryptohash_sha3_init(struct sha3_ctx *ctx, uint32_t hashlen)
 {
 	memset(ctx, 0, sizeof(*ctx));
 	ctx->hashlen = hashlen / 8;
 	ctx->bufsz = 200 - 2 * ctx->hashlen;
 }
 
-void sha3_update(struct sha3_ctx *ctx, uint8_t *data, uint32_t len)
+void cryptohash_sha3_update(struct sha3_ctx *ctx, uint8_t *data, uint32_t len)
 {
 	uint32_t to_fill;
 
@@ -135,7 +135,7 @@ void sha3_update(struct sha3_ctx *ctx, uint8_t *data, uint32_t len)
 	}
 }
 
-void sha3_finalize(struct sha3_ctx *ctx, uint8_t *out)
+void cryptohash_sha3_finalize(struct sha3_ctx *ctx, uint8_t *out)
 {
 	uint64_t w[25];
 
